

{=======================================================================

                         ASYNC

               Turbo Pascal Asynch Functions

========================================================================}

Type tComPort =  (Com1, Com2, Com3, Com4, Com5 );
     tBaud = (b110, b150, b300, b600, b1200, b2400, b4800, b9600);
     tParity = (pSpace, pOdd, pMark, pEven, pNone);
     tDatabits = (d5, d6, d7, d8);
     tStopbits = (s1, s2);

                         {  Define address adders for the various
                             Async card registers.                    }
Const RBR = $00;         { xF8   Receive Buffer Register             }
      THR = $00;         { xF8   Transmitter Holding Register        }
      IER = $01;         { xF9   Interrupt Enable Register           }
      IIR = $02;         { xFA   Interrupt Identification Register   }
      LCR = $03;         { xFB   Line Control Register               }
      MCR = $04;         { xFC   Modem Control Register              }
      LSR = $05;         { xFD   Line Status Register                }
      MSR = $06;         { xFE   Modem Status Register               }
      DLL = $00;         { xF8   Divisor Latch Least Significant     }
      DLM = $01;         { xF9   Divisor Latch Most  Significant     }
      imlist1: array[Com1..Com5] of integer = ($EF, $F7, $FB, $FB, $FB);{ ASynch Interrupt Masks }
      imlist2: array[Com1..Com5] of integer = ($FF, $FF, $FB, $F7, $EF);{ ASynch Interrupt Masks }
      ivlist:  array[Com1..Com5] of integer = ($000C, $000B, $0072, $0073, $0074);{  ASynch hardware interrupt addresses }
      PICCMD1 = $20;           {  8259 Priority Interrupt Controller  }
      PICMSK1 = $21;           {  8259 Priority Interrupt Controller  }
      PICCMD2 = $A0;           {  8259 Priority Interrupt Controller  }
      PICMSK2 = $A1;           {  8259 Priority Interrupt Controller  }
      ComBaseAddr : array[Com1..Com5] of integer = ($3F8,$2F8,$3E8,$3E0, $2F0);
      baudcode: array[b110..b9600] of integer = ($417, $300, $180, $C0, $60, $30, $18, $0C);
      paritycode: array[pSpace..pNone] of byte  = ($38, $08, $28, $18, $00);
      databitscode: array[d5..d8] of byte = ($00, $01, $02, $03);
      stopbitscode: array[s1..s2] of byte = ($00, $04);
      baudrate     :  array[b110..b9600] of integer
            = (110,150,300,600,1200,2400,4800,9600);

      MaxRingBuff = $7FFF;
var
      ringbuf : array[0..MaxRingBuff] of char;
      datardy       : boolean;
      readptr       : integer;
      writptr       : integer;
      ComBase_TXRDY : boolean;
      ComBase_CTS   : boolean;
      ComSaveVec : array[Com1..Com5] of Pointer;

var      ComBase    : integer;
         Base       : tComPort;


 var ComBase_IIR, ComBase_MSR, ComBase_LSR : integer;

{-------This routine gets control upon an Asynch Interrupt---------------}
{F+}
Procedure Port_Interrupt; Interrupt;

Begin
  inline($FA);

  { Get the cause(es) of the interrupt - High Priority  }
  ComBase_IIR    := PORT[ComBase +    IIR];
  case ComBase_IIR of
        $02 : ComBase_TXRDY    := true;
        $04 : begin
                 ringbuf[writptr] := CHR(PORT[ComBase]);
                 writptr := (writptr + 1) and MaxRingBuff;
                 datardy := true;
              end;
        $00 : begin
                ComBase_MSR := PORT[ComBase +  MSR];
                ComBase_CTS := (ComBase_MSR and $80) = $80;
              end;
  end;

   { Service Received data - Low Priority }
   ComBase_LSR    := PORT[Combase    + LSR];

  PORT[PICCMD1] := $20;                 {  Send End Of Interrupt to 8259       }
  PORT[PICCMD2] := $20;
  inline($FB);
End;
{F-}

function hex(number : integer): string;
var    i : integer;
       k : integer;
     res : string;
const ch : array[0..15] of char =
 ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
begin
   res[0] := #2;
   for i := 1 to 2 do
   begin
     res[3-i] := ch[number and $0f];
     number := number shr 4;
   end;
   hex := res;
end;

function hex4(number : integer): string;
var    i : integer;
       k : integer;
     res : string;
const ch : array[0..15] of char =
 ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
begin
   res[0] := #4;
   for i := 1 to 4 do
   begin
     res[5-i] := ch[number and $0f];
     number := number shr 4;
   end;
   hex4 := res;
end;


procedure setbaudrate(ComPort: tComport;
                      Baud:tBaud;
                      Parity: tParity;
                      Databits: tDatabits;
                      Stopbits: tStopbits);
var LCRreg   : byte;
    temp     : integer;
    ComBaseX : integer;
begin
  ComBaseX := ComBaseAddr[ComPort];
  LCRreg := $80;                                    {  Divisor Latch Access         }
  LCRreg := LCRreg or paritycode[Parity];           {  Setup Parity                 }
  LCRreg := LCRreg or databitscode[Databits];       {  Setup # data bits            }
  LCRreg := LCRreg or stopbitscode[Stopbits];       {  Setup # stop bits            }
  PORT[IER+ComBasex] := $00;                        {  Disable Data Avail interrupt }
  PORT[MCR+ComBasex] := $00;                        {  Restore Com interrupt vector }
  PORT[LCR+ComBaseX] := Port[LCR + ComBase] or  $80;{  Reset DLAB                   }
  PORT[DLM+ComBaseX] := Hi(baudcode[Baud]);         {  Set Baud rate                }
  PORT[DLL+ComBaseX] := Lo(baudcode[Baud]);         {  Set Baud rate                }
  PORT[LCR+ComBaseX] := LCRreg and $7F;             {  Reset DLAB                   }
  PORT[IER+ComBaseX] := $0F;                        {  Enable some interrupts       }
  PORT[MCR+ComBaseX] := $0B;                        {  Set RTS, DTR, OUT2           }
  delay(200);
end;


{-------------------- Open COM1 or COM2, a la Basic -----------------}
Procedure OpenCom(ComPort: tComPort;
                  Baud: tBaud;
                  Parity: tParity;
                  Databits: tDatabits;
                  Stopbits: tStopbits);

var ComBaseX : integer;

Begin
  inline($FA);
  ComBaseX := ComBaseAddr[ComPort];           {  Select Input Port        }
  if (port[ComBaseX+IER] <> 0) then
  begin
    PORT[IER+ComBasex] := $00;              {  Disable Data Avail interrupt }
    PORT[MCR+ComBasex] := $00;              {  Restore Com interrupt vector }
  {  Halt;  }
  end;
  PORT[MCR+ComBaseX] := $00;              {  Restore Com interrupt vector }
  GetIntVec(ivlist[ComPort], ComSaveVec[ComPort]);
  SetIntVec(ivlist[Comport],@PORT_Interrupt);
  readptr := 0;                              {  Init buffer pointers     }
  writptr := 0;                              {  Init buffer pointers     }
  datardy := false;
  ComBase_TXRDY    := false;
  PORT[PICMSK1] := PORT[PICMSK1] and imlist1[Comport];{  Enable ASynch Int }
  PORT[PICMSK2] := PORT[PICMSK2] and imlist2[Comport];{  Enable ASynch Int}
  setbaudrate(ComPort,Baud,Parity,Databits,Stopbits);
  PORT[IER+ComBaseX] := $03;                  {  Enable only receive  }
  PORT[MCR+ComBaseX] := $0B;                  {  Set RTS, DTR, OUT2       }
  PORT[PICCMD1] := $20;                 {  Send End Of Interrupt to 8259       }
  PORT[PICCMD2] := $20;
  inline($FB);
End;


{---------------- Close any initialized COM -------------------------}
Procedure CloseCom(ComPort : tComPort);
var ComBasex : integer;
Begin
  ComBasex := CombaseAddr[ComPort];
  PORT[PICMSK1] := PORT[PICMSK1] or ($FF - imlist1[Comport]); { Disable interrupt }
  PORT[PICMSK2] := PORT[PICMSK2] or ($FF - imlist2[Comport]); { Disable interrupt }
  PORT[IER+ComBasex] := $00;              {  Disable Data Avail interrupt }
  PORT[MCR+ComBasex] := $00;              {  Restore Com interrupt vector }
  SetIntVec(ivlist[ComPort], ComSaveVec[ComPort]);
End;

function get_rcv_char: char;
begin
  get_rcv_char := ringbuf[readptr];
  readptr := (readptr + 1) and MaxRingBuff;
  inline($FA);
  if (readptr = writptr) then datardy := false;
  inline($FB);
end;

Function  recvchar: char;
Begin
  If datardy then recvchar := char(ord(get_rcv_char) and $7f);
end;



procedure purgeline;
var temp : char;
begin
 repeat
   if datardy then temp := recvchar;
   delay(10);
 until not datardy;
end;

function inready: boolean;
begin
  inready := keypressed or datardy;
end;

function outready: boolean;
begin
 outready := ComBase_TXRDY;  {( (PORT[LSR+ComBase] and $20) <> 0); }
end;


procedure ClearBuffer;
var i : integer;
begin
    for i := $00 to $7FFF do ringbuf[i] := #00;
    purgeline;
    datardy := false;
    readptr  := 0;
    writptr := 0;
end;


function cts_temp: boolean;
  begin
   { ComBase_MSR := PORT[Combase + MSR]; }
    cts_temp := ComBase_CTS;   { ((ComBase_MSR and $80) = $80) or local; }
  end;


{This function returns true if a carrier tone is present on the modem
 and is frequently checked to see if the caller is still present.
 It always returns "true" in the local mode.  The delays also provide for
 some of the spike noise that is present on telephone lines.......}
var cts_count : integer;
function cts: boolean;
  var cts_temp_var : boolean;
  begin
    cts_temp_var := cts_temp;
    if cts_temp_var then cts_count := 0 else cts_count := cts_count + 1;
    if ((not cts_temp_var) and (cts_count < 5)) then
    begin
       delay(30);
       cts_temp_var := cts_temp;
    end;
    cts := cts_temp_var;
  end;

  procedure xmitchar(ch : char);
Var LSRreg: byte;
  Begin
     {Repeat LSRreg := (PORT[LSR+ComBase] and $20) Until LSRreg <> 0; }
     Repeat until ComBase_TXRDY ;
     ComBase_TXRDY := false;
     PORT[THR+ComBase] := ORD(ch);
 end;


procedure xmitchar1(ch : char);
Var LSRreg: byte;
  Begin
  if (cts) then
  begin
     {Repeat LSRreg := (PORT[LSR+ComBase] and $20) Until LSRreg <> 0; }
     Repeat until ComBase_TXRDY or (not cts);
     ComBase_TXRDY := false;
     PORT[THR+ComBase] := ORD(ch);
  end;
 { WindowsMessageHandler; }
end;

 procedure xmit(x:byte);
    begin
      xmitchar(chr(x));
    end;

  function inbyte: byte;
    var temp: char;
    begin
      repeat until inready or not cts;
      if keypressed then temp := readkey else
          if datardy then temp := get_rcv_char;
      inbyte := ord(temp);
    end;



procedure dropRTS;
{ Lowers RS-232 RTS line - used to inhibit auto-answer
   and to cause modem to hang up }
  begin
      PORT[MCR+ComBase] := $00;    {  Reset RTS, DTR, OUT2       }
  end;

procedure raiseRTS;
(* Raises RTS line to enable auto-answer *)
  begin
    PORT[MCR+ComBase] := $0B;      {  Set RTS, DTR, OUT2       }
  end;



